package slimeknights.tconstruct.tools.client;

import com.google.common.collect.ImmutableList;

import net.minecraft.block.Block;
import net.minecraft.block.BlockChest;
import net.minecraft.block.BlockEnderChest;
import net.minecraft.block.BlockSign;
import net.minecraft.block.BlockSkull;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Gui;
import net.minecraft.client.multiplayer.PlayerControllerMP;
import net.minecraft.client.renderer.BlockRendererDispatcher;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.client.resources.IResourceManagerReloadListener;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import org.lwjgl.opengl.GL11;

import java.util.List;

import slimeknights.tconstruct.library.tools.IAoeTool;
import slimeknights.tconstruct.library.tools.ToolCore;
import slimeknights.tconstruct.library.utils.ToolHelper;

@SideOnly(Side.CLIENT)
public class RenderEvents implements IResourceManagerReloadListener {

  private static final ResourceLocation widgetsTexPath = new ResourceLocation("textures/gui/widgets.png"); // GuiIngame.widgetsTexPath
  private final TextureAtlasSprite[] destroyBlockIcons = new TextureAtlasSprite[10];

  @SubscribeEvent
  public void onRenderGUI(RenderGameOverlayEvent.Pre event) {
    // we're only interested in the inventory
    if(event.type != RenderGameOverlayEvent.ElementType.HOTBAR) {
      return;
    }

    EntityPlayer player = Minecraft.func_71410_x().field_71439_g;
    ItemStack stack = player.func_71045_bC();
    if(stack == null) {
      return;
    }

    // do we hold a tool that supports secondary item usage?
    if(stack.func_77973_b() instanceof ToolCore && ((ToolCore) stack.func_77973_b()).canUseSecondaryItem()) {
      int slot = ToolHelper.getSecondaryItemSlot(player);

      if(slot != player.field_71071_by.field_70461_c) {
        // render the special border around the secondary item that would be used
        int x = event.resolution.func_78326_a() / 2 - 90 + slot * 20 + 2;
        int y = event.resolution.func_78328_b() - 16 - 3;

        // render a cool underlay thing
        GlStateManager.func_179131_c(1,1,1,0.5f);
        Minecraft.func_71410_x().func_110434_K().func_110577_a(widgetsTexPath);
        Gui.func_152125_a(x, y, 1, 23, 22, 22, 16, 16, 256,256);
      }
    }
  }

  @SubscribeEvent
  public void renderExtraBlockBreak(RenderWorldLastEvent event) {
    PlayerControllerMP controllerMP = Minecraft.func_71410_x().field_71442_b;
    EntityPlayer player = Minecraft.func_71410_x().field_71439_g;
    World world = player.field_70170_p;
    // AOE preview
    if(player.func_71045_bC() != null && player.func_71045_bC().func_77973_b() instanceof IAoeTool) {
      MovingObjectPosition mop = player.func_174822_a(controllerMP.func_78757_d(), event.partialTicks);
      if(mop != null) {
        ItemStack stack = player.func_71045_bC();
        ImmutableList<BlockPos> extraBlocks = ((IAoeTool) stack.func_77973_b()).getAOEBlocks(stack, world, player, mop
            .func_178782_a());
        for(BlockPos pos : extraBlocks) {
          event.context.func_72731_b(player, new MovingObjectPosition(new Vec3(0,0,0), null, pos), 0, event.partialTicks);
        }
      }
    }

    // extra-blockbreak animation
    if(controllerMP.field_78778_j) {
      if(controllerMP.field_85183_f != null &&
         controllerMP.field_85183_f.func_77973_b() instanceof IAoeTool &&
         ((IAoeTool) controllerMP.field_85183_f.func_77973_b()).isAoeHarvestTool()) {
        ItemStack stack = controllerMP.field_85183_f;
        BlockPos pos = controllerMP.field_178895_c;
        drawBlockDamageTexture(Tessellator.func_178181_a(),
                               Tessellator.func_178181_a().func_178180_c(),
                               player,
                               event.partialTicks,
                               world,
                               ((IAoeTool) stack.func_77973_b()).getAOEBlocks(stack, world, player, pos));
      }
    }
  }

  // RenderGlobal.drawBlockDamageTexture
  public void drawBlockDamageTexture(Tessellator tessellatorIn, WorldRenderer worldRendererIn, Entity entityIn, float partialTicks, World world, List<BlockPos> blocks)
  {
    double d0 = entityIn.field_70142_S + (entityIn.field_70165_t - entityIn.field_70142_S) * (double)partialTicks;
    double d1 = entityIn.field_70137_T + (entityIn.field_70163_u - entityIn.field_70137_T) * (double)partialTicks;
    double d2 = entityIn.field_70136_U + (entityIn.field_70161_v - entityIn.field_70136_U) * (double)partialTicks;

    TextureManager renderEngine = Minecraft.func_71410_x().field_71446_o;
    int progress = (int)(Minecraft.func_71410_x().field_71442_b.field_78770_f*10f) - 1; // 0-10

    if(progress < 0)
      return;

      renderEngine.func_110577_a(TextureMap.field_110575_b);
      //preRenderDamagedBlocks BEGIN
      GlStateManager.func_179120_a(774, 768, 1, 0);
      GlStateManager.func_179147_l();
      GlStateManager.func_179131_c(1.0F, 1.0F, 1.0F, 0.5F);
      GlStateManager.func_179136_a(-3.0F, -3.0F);
      GlStateManager.func_179088_q();
      GlStateManager.func_179092_a(516, 0.1F);
      GlStateManager.func_179141_d();
      GlStateManager.func_179094_E();
      //preRenderDamagedBlocks END

      worldRendererIn.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
      worldRendererIn.func_178969_c(-d0, -d1, -d2);
      worldRendererIn.func_78914_f();

      for(BlockPos blockpos : blocks) {
        double d3 = (double)blockpos.func_177958_n() - d0;
        double d4 = (double)blockpos.func_177956_o() - d1;
        double d5 = (double)blockpos.func_177952_p() - d2;
        Block block = world.func_180495_p(blockpos).func_177230_c();
        TileEntity te = world.func_175625_s(blockpos);
        boolean hasBreak = block instanceof BlockChest || block instanceof BlockEnderChest
                           || block instanceof BlockSign || block instanceof BlockSkull;
        if (!hasBreak) hasBreak = te != null && te.canRenderBreaking();

        if (!hasBreak)
        {
            IBlockState iblockstate = world.func_180495_p(blockpos);

            if (iblockstate.func_177230_c().func_149688_o() != Material.field_151579_a)
            {
              TextureAtlasSprite textureatlassprite = this.destroyBlockIcons[progress];
              BlockRendererDispatcher blockrendererdispatcher = Minecraft.func_71410_x().func_175602_ab();
              blockrendererdispatcher.func_175020_a(iblockstate, blockpos, textureatlassprite, world);
            }
        }
      }

      tessellatorIn.func_78381_a();
      worldRendererIn.func_178969_c(0.0D, 0.0D, 0.0D);
      // postRenderDamagedBlocks BEGIN
      GlStateManager.func_179118_c();
      GlStateManager.func_179136_a(0.0F, 0.0F);
      GlStateManager.func_179113_r();
      GlStateManager.func_179141_d();
      GlStateManager.func_179132_a(true);
      GlStateManager.func_179121_F();
      // postRenderDamagedBlocks END
  }

  @Override
  public void func_110549_a(IResourceManager resourceManager) {
    TextureMap texturemap = Minecraft.func_71410_x().func_147117_R();

    for (int i = 0; i < this.destroyBlockIcons.length; ++i)
    {
      this.destroyBlockIcons[i] = texturemap.func_110572_b("minecraft:blocks/destroy_stage_" + i);
    }
  }
}
